# jQuery Form Validator
A complete way to validate forms.
It can go from **easy** to **advanced** level of complexity.

## Getting Started

These instructions will get you an idea of **how to use jQuery Form Validator**.

### Prerequisites

Like any other jQuery plugin:

```
Put 'jquery-form-validator.js' in your Project folder.
```

Then import into HTML page at **the end of <body> tag**.

```html
<body>
...
<script src="form-validator.js"></script>
</body>
```

## Using Form Validator

### Make a simple Form.

<details><summary>Click to Show</summary>
<p>

Create a form inside <body> tag.

```html
<form novalidate id="myform" onsubmit="return validate(this);">
	<div>
		<label>Name:</label>
		<input id="fname" type="text">
		<span id="vfname"></span>
		<br>
		<label>Password:</label>
		<input id="fpassword" type="password" autocomplete="off">
		<span id="vfpassword"></span>
		<br>
		<label>E-mail:</label>
		<input id="femail" type="email">
		<span id="vfemail"></span>
		<br>
		<label>Cellphone:</label>
		<input id="fphone" type="tel">
		<span id="vfphone"></span>
	</div>
	<div>
		<input type="submit" value="Submit">
	</div>
</form>
```
</p>
</details>

### Create a JS Object using the form properties.

<details><summary>Click to Show</summary>
<p>

Put a new Script after importing 'jquery-form-validator.js'.
Here is an example:

```javascript
<script>
const formsToValidate = [{
	formId: "myform",
	inputs: [{
			inputId: "fname",
			alertId: "vfname",
			minLength: 3,
			maxLength: 21, //If user overrides length limit, then the default lengthMessage will appear.
			blockSpace: true,
			required: true,
			//It has no 'regex', for example.
			lengthMessage: 'Name must be between %min and %max characters!', //Custom lengthMessage.
			blockSpaceMessage: "Spaces are not allowed.",
			listener: function (valid, errors, regexStatus) {
				$('#fname').css('background-color', valid ? '#85e085' : '#ffb3b3');
			}
			//It has no 'validatedMessage', for example.
		},
		{
			inputId: "fpassword",
			//It has no 'alertId', for example.
			minLength: 7,
			maxLength: 25,
			blockSpace: true,
			required: true,
			regex: [
				[1, "STRONG_PASSWORD", "^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[!@#\$%\^&\*])(.{%min,%max})"], //Must be between 'min' and 'max' length, one of A-Z characters, one of a-z, one of 1-9, one of special (!@#$%^&*).
				[0, "MEDIUM_PASSWORD", "^(((?=.*[a-z])(?=.*[A-Z]))|((?=.*[a-z])(?=.*[0-9]))|((?=.*[A-Z])(?=.*[0-9])))(.{%min,%max})"], //Must be between 'min' and 'max' length, one of A-Z characters or one of a-z, one of 1-9 or one of special (!@#$%^&*).
				[-1, "WEAK_PASSWORD"]
			],
			listener: function (valid, errors, regexStatus) {
				if (errors.includes('REQUIRED'))
					$('#vfpassword').text('\u2022 This camp is required!');
				else if (regexStatus[0] == 1) {
					$('#vfpassword').text('\u2022 Strong password. You\'re safe!');
					$('#fpassword').css('background-color', '#5dd55d');
				} else if (regexStatus[0] == 0) {
					$('#vfpassword').text('\u2022 Medium password. Include both upcase/lowcase letters, numbers and symbols to make it stronger.');
					$('#fpassword').css('background-color', '#85e085');
				} else if (regexStatus[0] == -1) {
					$('#vfpassword').text('\u2022 Weak password. Must contains 7-25 characters (at least one letter and/or a number and/or a symbol).');
					$('#fpassword').css('background-color', '#ffb3b3');
				}
			}
			//It has none custom messages due to our listener function, for example.
		},
		{
			inputId: "femail",
			alertId: "vfemail",
			minLength: 5,
			maxLength: 100,
			blockSpace: true,
			required: true,
			customFilter: function (input) {
				filteredErrors = [];
				if (input.toLowerCase() == 'abc@test.com')
					filteredErrors.push('EMAIL_ALREADY_USED');
				return filteredErrors;
			},
			listener: function (valid, errors, regexStatus) {
				if (!valid && errors.includes('EMAIL_ALREADY_USED'))
					$('#vfemail').text('This e-mail has already used! Please, insert another address.');
				//Our listener function does not changes the input background-color, for example.
			},
			regexMessage: 'Please enter a valid e-mail!',
			validatedMessage: "Valid e-mail!"
		},
		{
			inputId: "fphone",
			alertId: "vfphone",
			minLength: 6,
			maxLength: 15,
			blockSpace: true,
			required: true,
			regex: [
				[1, "BR PHONE", "^(?:(55\\d{2})|\\d{2})[6-9]\\d{8}$"], //from https://www.regextester.com/94469
				[-1, "INVALID PHONE NUMBER"]
			],
			//It has no 'listener' or 'customFilter' function, for example.
			regexMessage: 'Please enter a valid BR cellphone number!',
			validatedMessage: "Valid cellphone!"
		}
	]
}];

prepareFormsToValidate(formsToValidate); // <-- Important to do it after declaring our 'formsToValidate' var.

function validate(formElement) { //This is the function inserted into our 'onsubmit' form attribute.
	return validateForm(getFormInfo(formElement.getAttribute('id'), null)); //Always do like that!
}
</script>
```
</p>
</details>

## Explaining the last lines

We've been prepared our Form Validator.
In order to close your Script, you need use '**prepareFormsToValidate(formsToValidate)**' method (passing our **last created object 'formsToValidate' as the parameter**).

```javascript
prepareFormsToValidate(formsToValidate);
   
function validate(formElement) {
	return validateForm(getFormInfo(formElement.getAttribute('id'), null));
}
```

It is **very important** to use our function 'validate' into our form 'onsubmit' attribute:

```html
<form novalidate id="myform" onsubmit="return validate(this);" method="POST">
```

## Detailed Object Explanation

|Input Property|Function|Example|Explanation|Required|Type|
|:-:|---|:-:|:-:|:-:|:-:|
|**inputId**|Sets the Input Element ID|inputId: "fname"|Specifies that the Form has an input "fname"|**Yes**|String|
|**alertId**|Sets the Alert Element that will receives **statuses** (like valid/invalid)|alertId: "vfname"|Specifies that the Form has an element "vfname", which will receive statuses|No|String|
|minLength|Sets the minimum length of an input|minLength: 3|_Self-explained_|No|Integer|
|maxLength|Sets the maximum length of an input|maxLength: 21|_Self-explained_|No|Integer|
|**blockSpace**|Remove spacement allowing of an input|blockSpace: true|Means that this input can't contains any spaces|No|Boolean|
|**regex**|Sets a Regex as validation rule for an input|_Example in another topic_|_Explained in another topic_|No|Array|
|**required**|Sets input as **required for validation**|required: true|Means that this input can't be undefined or null|No|Boolean|
|**customFilter**|Sets a **custom Function** that **is called every time the input is modified** (it is literally a 'custom filter')|_Example in another topic_|_Explained in another topic_|No|Function(**input**) that **returns a String Array**|
|**listener**|Sets a **custom Function** that **is called every time the input is checked**|_Example in another topic_|_Explained in another topic_|No|Function(**valid: boolean, errors: Array, regexStatus: Array**)|
|lengthMessage|Sets the **length error message** that will be displayed on **alertId** if it isn't undefined|lengthMessage: "Your password must be between %min and %max characters!"|_Self-explained. OBS:. Both '%min' and '%max' will be replaced by **minLength** and **maxLength**_|No|String|
|blockSpaceMessage|Sets the **space error message** that will be displayed on **alertId** if it isn't undefined|blockSpaceMessage: "This field can't contain spaces!"|_Self-explained_|No|String|
|regexMessage|Sets the **regex error message** that will be displayed on **alertId** if it isn't undefined|regexMessage: "Your password must have only A-z, 0-9 and !@#\$%\^&\* characters!"|_Self-explained_|No|String|
|requiredMessage|Sets the **required error message** that will be displayed on **alertId** if it isn't undefined|requiredMessage: "This field is required!"|_Self-explained_|No|String|
|**validatedMessage**|Sets the **successfully validated message** that will be displayed on **alertId** if it isn't undefined|validatedMessage: "Valid e-mail."|_Self-explained_|No|String|

### Regex

#### Regex Arrays are defined by the following example:

|ID (or Priority-Level)|Description|Regex|
|:-:|---|:-:|
|1|"STRONG_PASSWORD"|"^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[!@#\$%\^&\*])(.{%min,%max})"|
|0|"MEDIUM_PASSWORD"|"^(((?=.*[a-z])(?=.*[A-Z]))\|((?=.*[a-z])(?=.*[0-9]))\|((?=.*[A-Z])(?=.*[0-9])))(.{%min,%max})"|
|**-1**|"WEAK_PASSWORD"|_undefined, not needed_|

#### Regex Arrays Explanation:

|ID (or Priority-Level)|Description|Regex|
|:-:|---|:-:|
|0 or any positive number|_Insert your custom description here_|_Insert your Regex here_|
|**-1**|It means _REGEX_FAIL_, and must be the **last of array**. You can define this description as a Regex fail, like "INVALID_EMAIL", for example.|_undefined, not necessary_|

### Custom Filters

#### 'Custom Filters' are a Function called at the begin of validation process

Since a Custom Filter is a function, it will be called every time that the specified Input has been modified (content).
That function will be fired with the following paramater:

|Variable Name|Type|Example|Explanation|
|:-:|:-:|:-:|:-:|
|input|Text|"math.is.easy@drugs.com"|In this example, the user has typed his e-mail. You can now apply your own **validation rules** inside **customFilter** function|

**Is very important to return a String Array indicating custom errors:**
You need to create a new empty Array, then add your custom rules and errors.

<details><summary>Show a Full Custom Filter Example</summary><p>
</br>
It's interesting to **interact with a Listener too**.
</br>

```javascript
{
	inputId: "femail",
	alertId: "vfemail",
	minLength: 5,
	maxLength: 100,
	blockSpace: true,
	required: true,
	customFilter: function (input) {
		filteredErrors = []; //It's needed to created an empty array for the errors.
		if (input.toLowerCase() == 'abc@test.com') //Check in our simple system if this e-mail was already used
			filteredErrors.push('EMAIL_ALREADY_USED'); //If true, then add our custom error to the created Array.
		return filteredErrors;
	},
	listener: function (valid, errors, regexStatus) { //Listener Interaction!
		if (!valid && errors.includes('EMAIL_ALREADY_USED'))
			$('#vfemail').text('This e-mail has already used! Please, insert another address.');
	},
	regexMessage: 'Please enter a valid e-mail!',
	validatedMessage: "Valid e-mail!"
}
```
![sc](https://user-images.githubusercontent.com/10366880/53147060-276eb500-3585-11e9-988f-ece3547d5930.png)

</p>
</details>

</br>

### Listeners

#### Listeners are a Function called at the end of validation process

Since a Listener is a function, it will be called every time that the specified Input has been modified & checked (content).
That function will be fired with the following parameters:

|Variable Name|Type|Output Example|Explanation|
|:-:|:-:|:-:|:-:|
|valid|Boolean|true|Form has been validated|
|errors|Array|["UNDER_MINIMUM_LENGTH", "REQUIRED"]|Form has not been validated since errors exists|
|regexStatus|Array|[0, "VALID_EMAIL"]|Statuses based on Regex tests. **Status Number indicates validated (>=0) or invalidated (=-1)**|

Notice that the **Regex Status example** shows **0 (which is not -1)**, indicating that the Regex was succeeded.</br>
If the **Regex Status** shows **-1**, it indicates that the Regex was failed.</br>
Concluding: if the **status** shows **0** and **NONE_REGEX**, it is valid (means that you hasn't defined a Regex).</br>

<details><summary>Show Listener Function example</summary><p>

```javascript
{
	inputId:...,
	minLength:...,
	maxLength:...,
	regex:...,
	listener: function (valid, errors, status) {
		$('#vfpassword').text(`[Valid: ${valid}], [Errors: ${errors}], [Statuses: ${status}]`);
	}
}
```

</p>
</details>

</br>
Done.

## Author

* **Guilherme Rodrigues** - *JavaScript contribution* - [RodDevelopment](https://github.com/RodDevelopment)
